! This file is part of SusHi.
!
! It includes initialization routines.
!

C-{{{ initsushi

      subroutine initsushi(htl_in,fhflagout,thdmcflagout,delmb_out,
     &extflag)
      implicit none
      double precision delmb_out,ms42
      integer htl_in
      logical fhflagout,thdmcflagout,extflag

      double precision thdmcparams(15)!, thdmcres(15)
      integer slha
      integer thdmc_set_param
      
      include '../commons/common-keys.f'
      include '../commons/common-consts.f'
      include '../commons/common-citations.f'
      include '../commons/common-inputoutput.f'
      include '../commons/common-quark.f'
      include '../commons/common-vars.f'
      include '../commons/common-int.f'
      include '../commons/common-ren.f'

      SUversion='   1.3.0   '
      SUdate=   ' Apr 2014 '

      pi = 3.14159265358979323846264338328d0
      z2 = 1.6449340668482264364724d0
      z3 = 1.2020569031595942853997d0
      z4 = 1.0823232337111381915160d0
      z5 = 1.0369277551433699263314d0
      ca = 3.d0
      cf = 4/3.d0
      tr = 1/2.d0

      call citeinit()
      citations(1) = 1

      !read-in of parameters using readinsushi from inputoutput.f
      call readinsushi(fhflagout,thdmcflagout,extflag)

#ifndef CCH
      if (lcchkey) then
         call printinfosushi(6
     &        ,'SusHi was not compiled for cch production.')
         call printerrorsushi(1,6,'Use: make proc=cch')
      endif
#else
      call printinfosushi(6,'SusHi was compiled for cch production.')
      if (.not.lcchkey) then
         call printerrorsushi(1,6
     &        ,'Block SUSHI, entry 6 must be set to norder+10.')
      endif
      if (norderggh.ne.-1) then
         call printinfosushi(6
     &        ,'ggh production not possible with proc=cch')
         call printerrorsushi(1,6
     &        ,'Block SUSHI, entry 5 must be set to -1.')
      endif
      if (.not.ltotinc) then
         call printinfosushi(6
     &        ,'Only total inclusive cross section implemented for'/
     &        /' cch.')         
         call printerrorsushi(1,6
     &        ,'Block DISTRIB, entries 1,2,3 must be set to 0')
      endif
      if (model.ne.0) then
         call printinfosushi(6
     &        ,'Only SM cch production is implemented.')
         call printerrorsushi(1,6
     &        ,'Block SUSHI, entry 1 must be set to 0')
      endif
#ifdef FEYNHIGGS
      call printerrorsushi(1,6
     &     ,'Cannot set proc and predef flag simultaneously.')
#else
#ifdef THDMC
      call printerrorsushi(1,6
     &     ,'Cannot set proc and predef flag simultaneously.')
#endif
#endif
      call printinfosushi(6
     &     ,'Calculation ccbar -> h production!')
#endif

!     run FeynHiggs, if required:
      if (fhflagout) then
#ifndef FEYNHIGGS
         write(*,*) "SusHi: usage error"
         write(*,*) "SusHi is currently not linked to FeynHiggs,"
         write(*,*) "but input-file contains FEYNHIGGS block:"
         write(*,*) "please modify input file, or re-compile with"
         write(*,*) "make predef=FH"
         stop
#else
         citations(16) = 1
         citations(17) = 1
         citations(18) = 1
         citations(19) = 1
         call runfeynhiggs(alfasmz,mt,mbmb,mw,mz,mh,pseudo,delmb_out)
#endif
      end if

!     run 2HDMC, if required:
      if (thdmcflagout) then
#ifndef THDMC
         write(*,*) "Input file contains block 2DHMC,"
         write(*,*) "but SusHi was compiled without 2HDMC support."
         write(*,*) "Either re-compile with 'make predef=2HDMC',"
         write(*,*) "or remove 2HDMC Block from input file."
         stop
#else
         citations(25) = 1
         call runthdmc()
#endif
      endif

      if (Mh.Le.20.d0) then
         write(*,*) "Warning: You called SusHi with a small Higgs mass."
         write(*,*) "Results might not be reasonable."
         write(*,*) "Electroweak contributions not implemented."
c       stop
      endif
      if (Mh.Ge.1500.d0) then
         write(*,*) "Warning: You called SusHi with a large Higgs mass."
         write(*,*) "Results might not be reasonable."
         write(*,*) "Electroweak contributions not implemented."
c       stop
      end if

      q0 = q0c + q01*Mstop1 + q02*Mstop2

      !determine whether to use the heavy top limit 
      !m_t -> inf in the top-cont.
      if(htl_in.eq.0) then
         htl = .false.
      else
         htl = .true.
      endif

      !call ffini

      !settings in case of a fourth generation
      if((yukfac(4).ne.0.d0).or.(yukfac(5).ne.0.d0)) then

         if(mbp.ne.mtp) then
            write(*,105)
            write(*,*) 'Warning: masses of the 4th generation quarks '
            write(*,*) 'are not equal. Setting m_b` = m_t`. '
            write(*,105)
            mbp = mtp
         endif

         ms42 = MSUSY**2+mtp**2
         mstp12 = ms42 - MSUSY**2
         mstp22 = ms42 + MSUSY**2
         msbp12 = mstp12
         msbp22 = mstp22
         mstp1 = dsqrt(mstp12)
         mstp2 = dsqrt(mstp22)
         msbp1 = dsqrt(msbp12)
         msbp2 = dsqrt(msbp22)

         Mh = dsqrt(
     &          Mh**2
     &        + (3/2.d0 * dlog(ms42/Mbp**2) * Mbp**4
     &        +  3/2.d0 * dlog(ms42/Mtp**2) * Mtp**4 ) 
     &        / (Pi/dsqrt(dsqrt(2.d0)*GF))**2 )

         if(Mh.gt.350.d0) then
            write(*,105)
            write(*,*) "Error: Higgs mass too large.", Mh
            write(*,105)
            stop
         endif
      endif

      mst12 = Mstop1**2
      mst22 = Mstop2**2     
      mcmc = Mcharm
      mbp2 = Mbp**2
      mtp2 = Mtp**2
      msb12 = Msbot1**2
      msb22 = Msbot2**2     

      !definition of renormalization scale and variables
      muR = Mh * muRfactor
      muF = Mh * muFfactor

      if (mbrunyuk.eq.1) then
         muB = mbmb
      else if (mbrunyuk.eq.2) then
         muB = muR
      else
         muB = Mh
      end if

      mh2 = Mh**2
      z = mh2/cme**2
      sigmanull = GF/288/dsqrt(2.d0)/Pi
     &     * 0.38937966d9 ! last factor: conversion gev -> pb
      nf = 5.d0
      if (lcchkey) nf = 4.d0
      betanull = 11/12.d0*ca-nf/6.d0
      ln2 = dlog(2.d0)
      lfh = dlog(muF**2/mh2)
      lfr = dlog(muF**2/muR**2)
      lrt = dlog(muR**2/mt**2)
      
      call SUSHI_BERNINI(18)

 105  format('#--------------------------------------------------#')
      end

C-}}}
C-{{{ initcouplings

      subroutine initcouplings(fhflagout,delmb,delAb)
      implicit none
      double precision cthetab4,sthetab4,cthetat4,sthetat4
      double precision delmb,delAb
      double complex ALOSMh,ALOSMA,ALOSUSYh
      logical fhflagout

      include '../commons/common-inputoutput.f'
      include '../commons/common-quark.f'
      include '../commons/common-vars.f'
      include '../commons/common-ren.f'
      include '../commons/common-readdata.f'

      gt3 = 1.d0
      gb3 = 1.d0

      !settings for a fourth generation
      if((yukfac(4).ne.0.d0).or.(yukfac(5).ne.0.d0)) then

         cthetat4 = 0.d0
         sthetat4 = 1.d0
         cthetab4 = 0.d0
         sthetab4 = 1.d0

         call quarkhiggscoup(beta,alpha,gt4,gb4,model,twohdmver,pseudo)

         call evalcsusy(Mbp,msbp1,msbp2,Mtp,Mstp1,Mstp2,Mgl,beta,
     &Mw,Mz,alpha,muR,muSUSY,cthetat4,sthetat4,cthetab4,sthetab4,
     &c1sm14,c1sm24,c1sm34,c1susy1t4,c1susy2t4,c1susy1b4,c1susy2b4,
     &gb14,gbh124,gbh214,gb24,gt14,gth124,gth214,gt24,pseudo)

         gbpe  = (c1susy1b4-gb4*c1sm14) * yukfac(9)
         gbpe2 = (c1susy2b4-gb4*c1sm24) * yukfac(9)

         gtpe  = (c1susy1t4-gt4*c1sm14) * yukfac(8)
         gtpe2 = (c1susy2t4-gt4*c1sm24) * yukfac(8)

         gbp = gb4 * yukfac(5)
         gtp = gt4 * yukfac(4)
         gbp1 = gb14 * yukfac(9)
         gbp2 = gb24 * yukfac(9)
         gtp1 = gt14 * yukfac(8)
         gtp2 = gt24 * yukfac(8)

      else

         gbp = 0.d0
         gtp = 0.d0
         gbp1 = 0.d0
         gbp2 = 0.d0
         gtp1 = 0.d0
         gtp2 = 0.d0
         gbpe = 0.d0
         gbpe2 = 0.d0
         gtpe = 0.d0
         gtpe2 = 0.d0

      endif

      call getmass()

      if (mbossave.gt.0.D0) then
         mbos = mbossave
      end if

      gt = gt3*yukfac(2)

      comc1eff0rd = gt*(1.d0,0.d0)
      c1eff1rd = gt*2.75d0
      c1eff2rd = gt*(2777/288.d0 + 19/16.d0*lrt + nf*( -67/96.d0 +lrt
     &     /3.d0))
      c2eff1rd = 0.d0
      if (pseudo.eq.1) then
         c1eff1rd = 0.d0
         c1eff2rd = 0.d0
         c2eff1rd = gt*(-1/2.d0 + lrt)
      endif
      
      !MSSM and 2HDM
      if (model.eq.1) then
!Ab either taken from input file or FH
         Ab = Real(cAb)
         At = Real(cAt)
         beta = datan(tanb)
         call renormalize(fhflagout,delmb,delAb)
         q0 = q0c + q01*Mstop1 + q02*Mstop2
         call quarkhiggscoup(beta,alpha,gt3,gb3,model,twohdmver,pseudo)
         call evalcsusy(mbsb,msbot1,msbot2,mt,mstop1,mstop2,Mgl,beta,
     &        Mw,Mz,alpha,muR,muSUSY,cthetat,sthetat,cthetab,sthetab,
     &        c1sm13,c1sm23,c1sm33,c1susy1t3,c1susy2t3,c1susy1b3
     &        ,c1susy2b3,gb13,gbh123,gbh213,gb23,gt13,gth123,gth213,gt23
     &        ,pseudo)
         
         gb = gb3 * gb
         gc = gt3 * yukfac(1)
         gt = gt3 * yukfac(2)
         gb1 = gb13 * yukfac(7)
         gb2 = gb23 * yukfac(7)
         gt1 = gt13 * yukfac(6)
         gt2 = gt23 * yukfac(6)
         gte  = (c1susy1t3-gt3*c1sm13) * yukfac(6)
         gte2 = (c1susy2t3-gt3*c1sm23) * yukfac(6)
c.. this is input for ggh@nnlo:         
         !c1eff0rd = gt*c1sm13
         !replaced by exakt amplitude:
         if (pseudo.eq.1) then
         comc1eff0rd = mt2*gt*ALOSMA(mh**2,mt2)
         else
         comc1eff0rd = mt2*gt*ALOSMh(mh**2,mt2)
         end if
         !write(*,*) "c1eff0",gt*c1sm13,comc1eff0rd
         !write(*,*) "check",dsqrt(mt2),dsqrt(mst12)
         c1eff1rd = gt*c1sm23
         c1eff2rd = gt*c1sm33
         c2eff1rd = gt*c2eff1rd
         if (norderggh.ge.3) then
            !c1eff0rd = c1eff0rd + gte
            if (pseudo.ne.1) then
            comc1eff0rd = comc1eff0rd
     &                + mt2*(gt1*ALOSUSYh(mh**2,mst12)
     &                     + gt2*ALOSUSYh(mh**2,mst22))
            end if
            !write(*,*) "c1eff0-3",gt*c1sm13+gte,comc1eff0rd
            c1eff1rd = c1eff1rd + gte2
         endif
      endif
      
      if (model.eq.2) then
         beta = datan(tanb)
         call quarkhiggscoup(beta,alpha,gt3,gb3,model,twohdmver,pseudo)
         gb = gb3 * yukfac(3)
         gc = gt3 * yukfac(1)
         gt = gt3 * yukfac(2)
         comc1eff0rd = gt*comc1eff0rd
         c1eff1rd = gt*c1eff1rd
         c1eff2rd = gt*c1eff2rd
         c2eff1rd = gt*c2eff1rd
      endif     
      end

C-}}}
